# Python imports

# Lib imports

# Application imports
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk
from gi.repository import Gdk



class DynamicHeaderWidget(Gtk.Box):
    def __init__(self, move_callback, x, y):
        super(DynamicHeaderWidget, self).__init__()

        self._move_callback = move_callback
        self._current_x     = x
        self._current_y     = y
        self._drag_start_x  = 0
        self._drag_start_y  = 0
        self._prior_time    = 0
        self._update_block  = False

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        self.show_all()


    def _setup_styling(self):
        self.set_orientation(0)

    def _setup_signals(self):
        ...

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        self.is_dragging = False
        drag_eve    = Gtk.Button(label = "....")
        hresize_eve = Gtk.Button(label = "< >")

        drag_eve.set_hexpand(True)
        drag_eve.connect("button-press-event", self._press_event)
        drag_eve.connect("motion-notify-event", self._motion_event)
        drag_eve.connect("button-release-event", self._release_event)

        self.add(drag_eve)
        self.add(hresize_eve)

    def _press_event(self, widget = None, eve = None):
        self.is_dragging   = True
        self._drag_start_x = eve.x
        self._drag_start_y = eve.y

    def _motion_event(self, widget = None, eve = None):
        if self._update_block:
            self._drag_start_x = eve.x
            self._drag_start_y = eve.y
            return

        if self.is_dragging:
            offset_x = eve.x - self._drag_start_x
            offset_y = eve.y - self._drag_start_y

            self._current_x += offset_x
            self._current_y += offset_y

            self._update_block = True
            self._move_callback(self.get_parent(), self._current_x, self._current_y)
            self._update_block = False

    def _release_event(self, widget = None, eve = None):
        self.is_dragging   = False
        self._drag_start_x = 0
        self._drag_start_y = 0
