# Python imports
import os

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

# Application imports
from ..sections_tab_widget import SectionsTabWidget
from ..page_widget import Page



class PagesException(Exception):
    ...


class Pages(Gtk.Notebook):
    def __init__(self, close_tab, label = None, section_manifest_pth = None):
        super(Pages, self).__init__()

        self._tab_widget  = SectionsTabWidget(self, close_tab, label)
        self._section_pth = os.path.dirname(section_manifest_pth)

        self._setup_styling()
        self._setup_signals()
        self._load_widgets()
        self._load_pages_data()


    def _setup_styling(self):
        self.set_tab_pos(1)  # NOTE:  LEFT = 0, RIGHT = 1, TOP = 2, BOTTOM = 3

    def _setup_signals(self):
        self.set_scrollable(True)

    def _load_widgets(self):
        start_box = Gtk.Box()
        end_box   = Gtk.Box()

        search = Gtk.SearchEntry()
        search.set_placeholder_text("Search...")
        search.connect("changed", self._text_search)

        add_btn = Gtk.Button()
        add_btn.set_image( Gtk.Image.new_from_icon_name("add", 4) )
        add_btn.set_always_show_image(True)
        add_btn.connect("released", self.create_page_view)

        start_box.add(search)
        start_box.add(add_btn)

        start_box.show_all()
        end_box.show_all()

        # PACKTYPE: 0 Start, 1 = End
        self.set_action_widget(start_box, 0)
        self.set_action_widget(end_box, 1)

    def _close_tab(self, button, page, eve = None):
        if self.get_n_pages() == 1:
            return

        page_num     = self.page_num(page)
        manifest_pth = os.path.dirname( page.get_manifest_pth() )
        msg          = f"Do you really want to delete the Section Page?"
        response     = event_system.emit_and_await("delete_files", (None, None, [manifest_pth,], msg))

        if response == -8:
            self.remove_page(page_num)


    def create_page_view(self, widget = None, eve = None, manifest_pth = None):
        page     = Page(self._close_tab, self._section_pth, manifest_pth)
        page_num = self.append_page(page, page.get_tab_widget())

        self.set_tab_detachable(page, False)
        self.set_tab_reorderable(page, True)

        page.show_all()
        self.set_current_page(page_num)

    def get_tab_widget(self):
        return self._tab_widget

    def get_section_pth(self):
        return self._section_pth

    def _text_search(self, widget = None, eve = None):
        ...

    def _load_pages_data(self):
        if not self._section_pth:
            raise PagesException("No section path provided...")

        path  = self._section_pth
        pages = settings.clean_pages_list( os.listdir(path) )

        logger.info(f"Loading Page parts from: {path}")
        if len(pages) == 0:
            settings.generate_page_manifest(path)

        pages = settings.clean_pages_list( os.listdir(path) )
        for page in pages:
            _page = os.path.join(path, page)
            manifest = os.path.join(_page, "MANIFEST")
            self.create_page_view(None, None, manifest)
